import {
  ProseEditor,
  Toolbar
} from 'substance'

import ContainerEditor from '../ContainerEditor'
import Comments from '../panes/Comments/CommentBoxList'
import CommentsProvider from '../panes/Comments/CommentsProvider'
import TrackChangesProvider from '../elements/track_change/TrackChangesProvider'
class MiniEditor extends ProseEditor {

  didMount () {
    const provider = this.getProvider()
    provider.config.miniEditorContext = this.getChildContext()
    provider.config.miniEditorSession = this.editorSession
    this.on('noteSelected', this.scrollTo, this)
    this.on('notes:updated', this.createNotes, this)
  }

  render ($$) {
    const el = $$('div').addClass('sc-mini-editor')
    let toolbar = this._renderMiniToolbar($$)
    let editor = this._renderEditor($$)
    let SplitPane = this.componentRegistry.get('split-pane')
    let ScrollPane = this.componentRegistry.get('scroll-pane')
    let Overlay = this.componentRegistry.get('overlay')

    var commentsPane = $$(Comments, {
      comments: this.props.comments || {},
      fragment: this.props.fragment,
      update: this.props.update,
      user: this.props.user
    }).addClass('sc-comments-pane')

    const editorWithComments = $$(SplitPane, { sizeA: '100%', splitType: 'vertical' })
      .append(
        editor,
        commentsPane
      )

    const contentPanel = $$(ScrollPane, {
      name: 'miniEditorContentPanel',
      scrollbarPosition: 'right'
    })
      .append(editorWithComments, $$(Overlay))
      .attr('id', 'mini-editor-content-panel')
      .ref('miniEditorContentPanel')

    el.append(
      $$(SplitPane, { splitType: 'horizontal' })
        .append(toolbar, contentPanel)
    )

    return el
  }

  _renderMiniToolbar ($$) {
    let commandStates = this.commandManager.getCommandStates()
    return $$('div').addClass('se-toolbar-wrapper').append(
      $$(Toolbar, {
        commandStates: commandStates,
        toolGroups: ['document', 'annotations']
      }).ref('mini_toolbar')
    )
  }

  _renderEditor ($$) {
    return $$(ContainerEditor, {
      book: this.props.book,
      comments: this.props.comments,
      containerId: this.props.containerId,
      editorSession: this.editorSession,
      disabled: this.props.disabled,
      history: this.props.history,
      fragment: this.props.fragment,
      spellcheck: 'native',
      trackChanges: this.props.trackChanges,
      user: this.props.user
    }).ref('mini_body')
  }

  scrollTo (nodeId) {
    this.refs.miniEditorContentPanel.scrollTo(nodeId)
  }

  createNodeData (note, index) {
    return {
      'type': 'isolated-note',
      'content': '',
      'parentNoteId': note.id,
      index: index
    }
  }

  createNotes () {
    const provider = this.getProvider()
    const notes = provider.computeEntries()
    console.log('notes Created', notes)
    const surface = this.getSurface()
    const container = surface.getContainer()
    for (let i = 0; i < notes.length; i++) {
      let nodeData = this.createNodeData(notes[i], i)
      this.editorSession.transaction(function (tx) {
        if (!tx.get(nodeData.id)) {
          let blockNode = tx.create(nodeData)
          tx.update(container.getContentPath(), { type: 'insert', pos: i, value: blockNode.id })
        }
      })
    }
  }

  getProvider () {
    return this.context.notesProvider
  }

  getSurface () {
    const containerId = this.props.containerId
    const provider = this.getProvider()
    return provider.config.miniEditorContext.surfaceManager.getSurface(containerId)
  }

  getChildContext () {
    const oldContext = super.getChildContext()
    const doc = this.doc

    // comments provider
    const commentsProvider = new CommentsProvider(doc, {
      commandManager: this.commandManager,
      comments: this.props.fragment.comments,
      containerId: this.props.containerId,
      controller: this,
      editorSession: this.editorSession,
      fragment: this.props.fragment,
      surfaceManager: this.surfaceManager,
      update: this.props.update
    })

    const trackChangesProvider = new TrackChangesProvider(doc, {
      commandManager: this.commandManager,
      containerId: this.props.containerId,
      controller: this,
      editorSession: this.editorSession,
      surfaceManager: this.surfaceManager,
      user: this.props.user
    })

    // attach all to context
    return {
      ...oldContext,
      commentsProvider,
      trackChangesProvider
    }
  }
}

export default MiniEditor
